import requests
from Common.fun_util import fun_util,DateEncoder
import allure
import json


class requsts_util:
    session = requests.session()
    def send_request(self,method,url,data,headers,**kwargs):
        method = str(method).lower()
        #rep = None
        try:
            if method == 'get':
                rep = requsts_util.session.request(method=method, url=url, params=data,headers = headers,**kwargs)
            else:
                if 'form' in str(headers):
                    rep = requsts_util.session.request(method=method, url=url, data=data,headers= headers, **kwargs)
                else:
                    rep = requsts_util.session.request(method=method, url=url, json=data,headers= headers, **kwargs)
        except:
            print('无法连接')
        else:
            if rep.status_code==200:
                resp = json.loads(rep.text)
                return resp
            else:
                print('请求状态异常：'+str(rep.status_code))





    def excute_interface(self,caseinfo1,domain):
        #循环读取yml中配置的接口参数
        for caseinfo in caseinfo1:
            #caseinfo是个dic，通过caseinfo.keys()获取key，使用list()转为list类型，取下标0即可，yml测试数据的动态管理
            caseinfo_key = list(caseinfo.keys())[0]
            #从config文件中读取domain与接口地址拼接,login接口可能用别的域名，判断一下

            url = domain + caseinfo[caseinfo_key]['path']
            #读取请求类型
            method = caseinfo[caseinfo_key]['method']
            #读取请求数据
            data = caseinfo[caseinfo_key]['data']
            #读取请求头
            headers = caseinfo[caseinfo_key]['headers']
            #读取断言类型
            assert_type = caseinfo[caseinfo_key]['assert_type']
            #读取断言信息
            is_assert = caseinfo[caseinfo_key]['is_assert']
            #读取描述
            description = caseinfo[caseinfo_key]['description']
            #发送请求
            resp = self.send_request(method=method,url=url,data=data,headers=headers)
            print('\n')
            print('描述：')
            print(description )
            with allure.step('请求url：'+url):
                print('\n'+'请求url：')
                print(url+'\n')
            with allure.step('请求header：'+json.dumps(headers,indent = 4,ensure_ascii=False)):
                print('请求header：')
                print(json.dumps(headers,indent = 4,ensure_ascii=False)+'\n')
            with allure.step('请求body：'+json.dumps(data,indent = 4,ensure_ascii=False,cls=DateEncoder)):
                print('请求body：')
                print(json.dumps(data,indent = 4,ensure_ascii=False,cls=DateEncoder)+'\n')
            with allure.step('返回：'+json.dumps(resp,indent = 4,ensure_ascii=False)):
                print('返回：')
                print(json.dumps(resp,indent = 4,ensure_ascii=False)+'\n')
            fun_util().check_assert(is_assert,resp,assert_type)

        print('----------------------------------------------------------------------------------------------------------')
        return resp



from airtest.core.api import *
import pytest
from Config.config import Mobile,Web
import allure
class MobileBasePage:
    #poco = None
    imagePath = Mobile().DaBao['imagePath']
    def locator(self, loc, type):
        if type == 'text':
            self.poco(text=loc).wait_for_appearance(timeout=10)
            with allure.step("元素定位-成功(text)：" + str(loc)):
                print("元素定位-成功(text)：" + str(loc))
            return True
        if type == 'name':
            self.poco(name=loc).wait_for_appearance(timeout=10)
            with allure.step("元素定位-成功(name)：" + str(loc)):
                print("元素定位-成功(name)：" + str(loc))
            return True
        if type == 'image':
            if exists(loc):
                with allure.step("元素定位-成功(image)：" + str(loc)):
                    print("元素定位-成功(image)：" + str(loc))
                return True
        else:
            with allure.step("不存在元素类型：类型为：" +type+"，属性为："+str(loc)):
                print("不存在元素类型：类型为：" +type+"，属性为："+str(loc))
            pytest.assume(False)
            return False
    def click(self, loc, type):
        if self.locator(loc, type):
            if type == 'text':
                self.poco(text=loc).click()
                with allure.step("点击元素(text)：" + str(loc)):
                    print("点击元素(text)：" + str(loc))
                return True

            if type == 'name':
                self.poco(name=loc).click()
                with allure.step("点击元素(name)：" + str(loc)):
                    print("点击元素(name)：" + str(loc))
                return True

            if type == 'image':
                touch(loc)
                with allure.step("点击元素(image)：" + str(loc)):
                    print("点击元素(image)：" + str(loc))
                return True

        else:
            with allure.step("点击元素-失败：" + str(loc) + type):
                print("点击元素-失败：" + str(loc) + type)
            pytest.assume(False)
            return False

    def setValue(self,loc,value,type):
        if self.locator(loc, type):
            if type == 'name':
                input_box = self.poco(name=loc)
                input_box.setattr("text", value)
                with allure.step("元素赋值-成功(name)：" + str(loc)):
                    print("元素赋值-成功(name)：" + str(loc))
            if type == 'text':
                input_box = self.poco(text=loc)
                input_box.setattr("text", value)
                with allure.step("元素赋值-成功(text)：" + str(loc)):
                    print("元素赋值-成功(text)：" + str(loc))
        else:
            with allure.step("元素赋值-失败：" + str(loc) + type):
                print("元素赋值-失败：" + str(loc) + type)
            pytest.assume(False)

    def inputValueAirtest(self,loc,value,type):
        if self.click(loc, type):
            if type == 'name':
                text(value)
                with allure.step("元素赋值-成功(name)：" + str(loc)):
                    print("元素赋值-成功(name)：" + str(loc))
            if type == 'text':
                text(value)
                with allure.step("元素赋值-成功(text)：" + str(loc)):
                    print("元素赋值-成功(text)：" + str(loc))
            if type == 'image':
                text(value)
                with allure.step("元素赋值-成功(image)：" + str(loc)):
                    print("元素赋值-成功(image)：" + str(loc))
        else:
            with allure.step("元素赋值-失败：" + str(loc) + type):
                print("元素赋值-失败：" + str(loc) + type)
            pytest.assume(False)

    def inputValuePoco(self,loc,value,type):
        if self.click(loc, type):
            if type == 'name':
                self.poco(name=loc).set_text(value)
                with allure.step("元素赋值-成功(name)：" + str(loc)):
                    print("元素赋值-成功(name)：" + str(loc))
            if type == 'text':
                self.poco(text=loc).set_text(value)
                with allure.step("元素赋值-成功(text)：" + str(loc)):
                    print("元素赋值-成功(text)：" + str(loc))
        else:
            with allure.step("元素赋值-失败：" + str(loc) + type):
                print("元素赋值-失败：" + str(loc) + type)
            pytest.assume(False)

class WebBasePage:
    imagePath = Web().xdz['imagePath']
    def openUrl(self,url):
        with allure.step("打开地址：" +url):
            print("打开地址：" +url)
            self.driver.get(self.url)
    #判断元素是否存在


    # 查找并返回元素
    def findElement(self,loc,type):
        try:
            if type == 'xpath' :
                with allure.step("发现元素:%s" % loc+',类型：xpath'):
                    print("发现元素:%s" % loc+',类型：xpath')
                element = self.driver.find_element_by_xpath(loc)
                return element
            # if type == 'xpath' :
            #     with allure.step("发现元素:%s" % loc+'xpath'):
            #         print("发现元素:%s" % loc+'xpath')
            #     element = self.driver.find_element_by_xpath(loc)
            #     return element
        except:
            with allure.step("发现元素失败:%s"  %(loc)):
                print("发现元素失败:%s"  %(loc))
            pytest.assume(False)
            return False




    def sendKeys(self,loc,type,value):
        self.findElement(loc,type).send_keys(value)
        with allure.step("输入:%s" % value):
            print("输入:%s" % value)

    def jsExcute(self,value):
        with allure.step("JS点击:%s" % value):
            print("点击:%s" % value )
            self.driver.execute_script(value)

    def click(self,loc,type):
        self.findElement(loc,type).click()
        with allure.step("点击:%s" % loc):
            print("点击:%s" % loc )

    def assertText(self,loc,type,text):
        if text in self.findElement(loc, type).text:
            with allure.step("文字断言:%s-成功" % text):
                print("文字断言:%s-成功" % text)
            pytest.assume(True)
        else:
            with allure.step("文字断言-失败:%s，实际为:%s" % (text,self.findElement(loc, type).text)):
                print("文字断言:%s-失败，实际为:%s" % (text,self.findElement(loc, type).text))
            pytest.assume(False)


import os
import yaml
from string import Template
import json
from Config.config import CommonConfig
class yaml_util:
    #读取extract.yml文件
    rootPath = CommonConfig.rootPath
    def read_extract_yaml(self,key):
        with open(self.rootPath+"/Common/common_var.yml",mode='r',encoding='utf-8') as f:
            value = yaml.load(stream=f,Loader=yaml.FullLoader)
            return value[key];


    def write_extract_yaml(self,data):
        file_path = os.path.join(self.rootPath, "Common", "common_var.yml")

        # 读取文件中的所有数据
        with open(file_path, mode='r', encoding='utf-8') as f:
            all_data = yaml.safe_load(f) or {}

        # 将传入的数据转换为字典格式，并更新到all_data中
        for d in data:
            for key_data in d.keys():
                if key_data in all_data:
                    all_data[key_data]=d[key_data]
                else:
                    all_data.update(d)


        # 将所有数据写入文件中
        with open(file_path, mode='w', encoding='utf-8') as f:
            yaml.dump(data=all_data, stream=f, allow_unicode=True)


    #清除extract.ym1文件
    def clean_extract_yaml(self) :
        with open(self.rootPath+"/Common/common_var.yml",mode='w',encoding='utf-8') as f:
            f.truncate()

    #读取测试用例的yml文件
    def read_testcase_yaml(self,yaml_name):
        with open(self.rootPath+yaml_name,mode='r',encoding='utf-8') as f:
            value = yaml.load(stream=f,Loader=yaml.FullLoader)
            return value;

    #写入测试用例的yml文件
    def write_testcase_yaml(self,caseinfo,content) :
        caseinfo_str = json.dumps(caseinfo)
        for cont in content:
            c = Template(caseinfo_str).safe_substitute(cont)
            caseinfo_str = c
        caseinfo_str = caseinfo_str.replace('"None"', 'null')
        caseinfo_list =json.loads(caseinfo_str)
        return caseinfo_list





import time
from random import random
import datetime
import random
import subprocess
import pytest
import allure
from PIL import Image
import ddddocr
from Config.config import CommonConfig,Interface,Web
import pymysql
import os
import hashlib
import requests
import json
from redis.client import StrictRedis
class fun_util:
    # def get_time(self):
    #     time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    #     return time

    def check_assert(self,expected, result, type):
        with allure.step('断言期望内容：' + json.dumps(expected, indent=4, ensure_ascii=False)+'；断言模式：'+type):
            print('断言期望内容：' + json.dumps(expected, indent=4, ensure_ascii=False)+'；断言模式：'+type)
        if result != None:
            if expected == None:
                with allure.step('断言结果：无须断言'):
                    print('断言结果：无须断言')
                    pytest.assume(True)
                    return True
            else:
                if type == 'and':
                    for expected_key, expected_value in expected.items():
                        # 取出的键值拼装新的单个字典
                        dic = dict.fromkeys([expected_key], expected_value)
                        # 字典转为字符串，并截取dic的花括号
                        dic = str(dic)
                        # 截取去除花括号
                        dic = dic[1:len(dic) - 1]
                        result = str(result)
                        # print(str(dic),str(result))
                        if dic in result :
                            continue
                        if dic not in result:
                            with allure.step('断言结果：断言失败'):
                                print('断言结果：断言失败')
                                pytest.assume(False)
                                return False
                    with allure.step('断言结果：断言成功'):
                        print('断言结果：断言成功')
                        pytest.assume(True)
                        return True
                if type == 'or':
                    for expected_key, expected_value in expected.items():
                        # 取出的键值拼装新的单个字典
                        dic = dict.fromkeys([expected_key], expected_value)
                        # 字典转为字符串，并截取dic的花括号
                        dic = str(dic)
                        # 截取去除花括号
                        dic = dic[1:len(dic) - 1]
                        result = str(result)
                        if dic in result:
                            with allure.step('断言结果：断言成功'):
                                print('断言结果：断言成功')
                                pytest.assume(True)
                                return True
                        if dic not in result:
                            continue
                    with allure.step('断言结果：断言失败'):
                        print('断言结果：断言失败')
                        pytest.assume(False)
                        return False
                if type == 'not_and':
                    for expected_key, expected_value in expected.items():
                        # 取出的键值拼装新的单个字典
                        dic = dict.fromkeys([expected_key], expected_value)
                        # 字典转为字符串，并截取dic的花括号
                        dic = str(dic)
                        # 截取去除花括号
                        dic = dic[1:len(dic) - 1]
                        result = str(result)
                        # print(str(dic),str(result))
                        if dic not in result:
                            continue
                        if dic  in result:
                            with allure.step('断言结果：断言失败'):
                                print('断言结果：断言失败')
                                pytest.assume(False)
                                return False
                    with allure.step('断言结果：断言成功'):
                        print('断言结果：断言成功')
                        pytest.assume(True)
                        return True
                if type == 'not_or':
                    for expected_key, expected_value in expected.items():
                        # 取出的键值拼装新的单个字典
                        dic = dict.fromkeys([expected_key], expected_value)
                        # 字典转为字符串，并截取dic的花括号
                        dic = str(dic)
                        # 截取去除花括号
                        dic = dic[1:len(dic) - 1]
                        result = str(result)
                        if dic not in result:
                            with allure.step('断言结果：断言成功'):
                                print('断言结果：断言成功')
                                pytest.assume(True)
                                return True
                        if dic in result:
                            continue
                    with allure.step('断言结果：断言失败'):
                        print('断言结果：断言失败')
                        pytest.assume(False)
                        return False
        else:
            pytest.assume(False)
            return False
            print('接口返回为空')

    #随机生成身份证
    def generate_id(self):
        # 地区码，可根据实际情况修改
        region_code = '110101'
        birth_year = str(random.randint(1950, 2015))
        birth_month = str(random.randint(1, 12)).rjust(2, '0')
        birth_day = str(random.randint(1, 28)).rjust(2, '0')
        sequence_code = str(random.randint(1, 999)).rjust(3, '0')
        id_17 = region_code + birth_year + birth_month + birth_day + sequence_code
        # 加权因子
        weights = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        # 校验码对应值
        check_codes = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2']
        # 对前17位数字依次乘以对应的加权因子并求和
        total = sum(int(id_17[i]) * weights[i] for i in range(17))
        check_digit = check_codes[total % 11]
        return id_17 + check_digit

    #随机生成车牌号
    def generate_license_plate(self):
        # 车牌号码由省份+字母+数字组成
        provinces = ["京", "津", "沪", "渝", "冀", "豫", "云", "辽", "黑", "湘", "皖", "鲁", "新", "苏", "浙", "赣",
                     "鄂", "桂", "甘", "晋", "蒙", "陕", "吉", "闽", "贵", "粤", "青", "藏", "川", "宁", "琼"]
        letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        numbers = "0123456789"

        province = random.choice(provinces)
        letter = random.choice(letters)
        number = "".join(random.choice(numbers) for _ in range(5))

        license_plate = province + letter + number
        return license_plate

    #向手机传输电脑图片
    def import_image(self,image_path, dest_path):
        command = f"adb push {image_path} {dest_path}"
        command1 =f"adb shell am broadcast -a android.intent.action.MEDIA_SCANNER_SCAN_FILE -d file://{dest_path}"
        subprocess.run(command.split(), stdout=subprocess.PIPE)
        subprocess.run(command1.split(), stdout=subprocess.PIPE)


    #
    # #UI自动化判断元素是否存在
    # def is_element_exist(self, driver,text):
    #
    #     max_retry = 5
    #     #driver.implicitly_wait(60)
    #     wait = WebDriverWait(driver, 10)
    #     wait.until(lambda driver: driver.page_source)
    #     source = driver.page_source
    #     # 循环重试
    #     for i in range(1,max_retry+1):
    #         time.sleep(1)
    #         #print(source)
    #         if text in source:
    #             print('找到元素：'+text)
    #             return True
    #         else:
    #             print('未找到元素：'+text+';进行第'+str(i)+'次重试')
    #             if i == max_retry:
    #                 print('未找到元素：'+text)
    #                 return False
    #新大宗验证码校验
    def verificationCode(self,driver,element):

        while True:
            driver.screenshot(Web().xdz['imagePath']+'printscreen.png')
            imgelement = driver.find_element_by_xpath(element)
            location = imgelement.location
            # 获取验证码的长宽
            size = imgelement.size
            # 写成我们需要截取的位置坐标
            rangle = (int(location['x']),
                      int(location['y']),
                      int(location['x'] + size['width']),
                      int(location['y'] + size['height']))
            i = Image.open(Web().xdz['imagePath']+'printscreen.png')
            fimg = i.crop(rangle)
            fimg = fimg.convert('RGB')
            # 保存我们截下来的验证码图片，并读取验证码内容
            fimg.save(Web().xdz['imagePath']+'code.png', quality=95, subsampling=0, compress_level=0)
            ocr = ddddocr.DdddOcr()
            with open(Web().xdz['imagePath']+'code.png', 'rb') as f:
                img_bytes = f.read()
                res = ocr.classification(img_bytes)
            print('原始验证码' + res)
            if len(res) == 4 and res.isdigit():
                print('识别正确'+res)
                break
            else:
                driver.execute_script(
                    "document.getElementsByClassName('code_image')[0].click()")
                time.sleep(1)
        print('返回正确验证码' + res)
        return res


    def excuteSql(sefl,host,user,password,db,filePath):
        conn = pymysql.connect(host=host, user=user, password=password, db=db)
        cursor = conn.cursor()

        # 读取 SQL 文件
        with open(CommonConfig.rootPath+filePath, 'r',encoding='utf-8') as f:
            sqlList = f.readlines()
            for sql in sqlList:
                print('执行sql：'+sql)
                cursor.execute(sql)
        conn.close()
    def video_result(self):
        trans_id = '6FEB23ACB0374985A2A52D282EDD5361u6643'
        app_security_key = 'ca61c3b7b9d50ae0406070c8758190b1'
        reward_name = '自动化测试'
        check_sign_raw = "%s:%s" % (app_security_key, trans_id)
        sign = hashlib.sha256(check_sign_raw.encode('utf-8')).hexdigest()

        # print(sign)
        url = "https://api.test.qy566.com/rest/receiveAdCallback?user_id=" + Interface().JiFen['user_id'] + "&trans_id=" + trans_id + "&reward_name=" + reward_name + "&reward_amount=10&extra={'source':'ios'}&sign=" + sign
        # url2 = "https://api.test.qy566.com/rest/receiveAdCallback?user_id=123&trans_id=5646282&reward_name=积分&reward_amount=1&extra={\"source\": \"android\"}&sign=18db1451c7b80f56d6c3d809ebb2ba983a1fbad90d3836687298257d54131b5b"
        res = requests.get(url=url)
        print(url)
        print(res.text)
        return json.loads(res.text)



import json
class DateEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        else:
            return json.JSONEncoder.default(self, obj)
